# The Catch 2023 -- Maritime Communications

## VPN Access

This one is nothing new nor complicated. Just download the configuration an start the vpn via `openvpn --config ctfd_ovpn.ovpn`. Depending on the setup, some additional steps might be needed to allow DNS names resolving over the VPN.

## Treasure map

In this case, having a big monitor or zoom in a little helps a lot. When you finally spot the "compression artifacts" (which shouldn't be there, it's a PNG file), you are on the right track. Changing the background color in GIMP will also improve the readability of the flag.

## Captain's coffee

Upon visiting the site, we quickly find out there is OpenAPI with a Swagger UI present. There are only two routes and we can quickly try them out. We can obtain a drink menu and use any id to order a drink with a flag.

## Ship web server

Provided server has self-signed certificate which should guide us to inspect it. Subject Alternative Name contains a list of other interesting sites we need to see. They are not in DNS but might be served on the same host. So either patch you `/etc/hosts` or use appropriate *Host* header to view the sites and extract the flag.

## Sonar logs

This one is a tough one. Not because it is hard, but because it is old. More to it later. Upon inspection, we immediately see a log file with events and times. Some of the events has depths (suspiciously also in hexadecimal format, that could be letters). A quick `grep -o '0x..' < sonar.log | xxd -r -p` will confirm this assumption. So we just need to sort the list. For that, we need to use the timestamps and take the timezone info into account. We can use some standard tools to work with that. For example, if we want to know the offset for the zone, `TZ=Asia/Hebron date +%z` will tell us. To convert the timestamps to something we can later sort on, we might use something like `date -d "$time $offset" +"%s"` or `TZ=America/Mazatlan date -d "2023-10-02 02:51:23" +"%s"`.

We can combine all the above to a nice and simple script:
```sh
while read -r; do
    zone=$(cut -f3 -d' ' <<<"$REPLY")
    time=$(cut -f1,2 -d' ' <<<"$REPLY")
    message=$(grep -o '0x..' <<<"$REPLY" | xxd -r -p)
    if [ -z "$message" ]; then
        message=\\t$(cut -f5- -d' ' <<<"$REPLY")
    fi
    date=$(TZ=$zone date -d "$time" +"%s")
    echo -e "$date $message"
done | sort
```

Now we can run the scripts against the sonar log and we have the flag. Well, almost. One letter is incorrect because in the meantime, in America/Mazatlan they decided to ditch DST switching this year and that's not reflected in the logs. Brute-forcing the correct solution is thankfully not that hard.

## Regular cube

A regular regular expression crossword. Just fill it up, it is well defined without any issues.

## Apha-Zulu quiz

Nice quiz with several good answers to one question, so one need to be careful. With a little bit of help from Wikipedia one can easily identify correct answers for all questions.

## Captain's password

This tasks is referencing to a recent vulnerability in KeePass. Searching for "KeePass crash dump" will quickly lead to an original PoC. The only challenge here is to compile the password dumper and find a machine to run the tools on.

## Signal flags

After a quick glance on the provided data, we can find out what is going on here. We need to read the ICS flags. We can quickly spot, that most of them are numerical (with some exceptions). After picking a random image, manually decoding the flags to see some hexadecimal values, decoding them to letters and obtaining string "Caletka?", we know we are good. If we count the images, we can confirm that we won't be doing this manually and we need a script.

As a first step, lets open some of the input images with a simple graphics editor and extract the important flags (just the hexadecimal values should be enough, you can extract all letters if you also want to read the "whole" story -- if the message does not start with 0, the letters use the ICS meaning; the country flags are redundant and need not to be extracted, I ended up with 26 *samples*).

From that, we can create a simple script to look up our *samples* inside the input and output the coordinates of the *sample* and its name (which corresponds to the letter/number on the *sample*). I decided to use opencv and python.

```py
import sys
import glob
import cv2 as cv
import numpy as np

img = cv.imread(sys.argv[1])
img_gray = cv.cvtColor(img, cv.COLOR_BGR2GRAY)

samples = {}
for data in glob.glob('data/*.png'):
    samples[data[5:-4]] = cv.imread(data, cv.IMREAD_GRAYSCALE)

targets = {}

for i in samples.keys():
    w, h = samples[i].shape[::-1]
    res = cv.matchTemplate(img_gray, samples[i], cv.TM_CCOEFF_NORMED)
    loc = np.where(res >= 0.99)

    for pt in zip(*loc[::-1]):
        targets[pt[1]] = i
        cv.rectangle(img, pt, (pt[0] + w, pt[1] + h), (0,0,255), 2)

for key in sorted(targets.keys()):
    print("%s" % targets[key], end="")

print()

cv.imwrite(sys.argv[2], img)
```

The script also outputs the picture showing all matches, so we can later inspect if we recognized all letters. So now we can decode the flags, but we also need to group the images (the same ship is there multiple times) and sort it by time. We can use the timestamp and ship id from the picture itself, `tesseract` will help to get the text data from the image.

We can put that together into a short script:

```sh
for i in input/signalization_*.png; do
    string=$(python detect.py "$i" "${i/input/output/}")
    prefix=$(cut -c-2 <<<"$string")
    text=$(tesseract -l eng "$i" - 2>/dev/null)
    timestamp=$(grep "Timestamp:" <<<"$text" | cut -f2- -d:)
    id=$(grep "Ship object ID:" <<<"$text" | cut -f2 -d:)
    if [ "0x" == "$prefix" ]; then
        output="$(xxd -r -p <<<"$string")"
    else
        output="$string"
    fi
    echo -e "$timestamp\t$id\t${i#input/}\t$output"
done
```

The resulting log contains all the ship-to-ship communication and after a while we can identify a suspicious communication containing base64 encoded string. We can decode it to get the flag.

## Cat code

This code needs a code review. After renaming some variables and functions, we can identify the important parts. The imports provide a very slow implementation of Fibonacci and a function which takes a Fibonacci number, takes some digits from it and interprets it as a chars. The main function just calls this with the number *770*; so not only we have a slow implementation, the result will not even fit into a reasonable data type.

```py
from meow import fib, meowmeow
def meoow():
    print(meowmeow(fib(770)))
```

Luckily, as the input is constant, we can switch to a very fast fib algorithm -> precompute the result. If we simplify and run the program, the flag is ours.

```py
def meowmeow(fib):
    fibo = "372389983027365429815575917206642213232212628235696698065740843380067225782522577028597273117715177446328592843112586047
07327062313057129673010063900204812137985"
    string = ''
    for elem in data:
        string = f"{string}{chr(int(''.join(fibo[m] for m in elem)))}"
    return string
```

## Component replacement

There is a simple IP based filter. We can experiment a little and try for example `curl -H 'X-Forwarded-For: 192.168.96.20' http://key-parts-list.cns-jv.tcc/`. We can immediately see that the server blindly trusts the header. Creating a simple script to try out all addresses from the range is trivial and leads to the flag.

## Naval chef's recipe

Depending on the tool one use to visit the site, this could be a very easy task. Visiting the web with a "normal" web-browser will yield in a redirection to HTTPS site with a "normal" content. However, just disabling the javascript (as the redirect is done just by manipulating `window.location`) or accessing the web with `curl` will immediately show the flag.
